import hashlib
import random
import string
from typing import NewType, Set, List, Tuple, NamedTuple

SALT_LENGTH = 16
# salt is random for each run
SECRET_SALT = "".join(
    random.choices(string.ascii_uppercase + string.digits, k=SALT_LENGTH)
)

SecretMapping = NewType("SecretMapping", List[Tuple[str, str]])


class ApiKeyConfig(NamedTuple):
    api_keys: Set[str]
    apikey_name: str


def create_secret_mapping(api_key_config: ApiKeyConfig) -> SecretMapping:
    mapping = SecretMapping([])
    for api_key in api_key_config.api_keys:
        query_param = f"{api_key_config.apikey_name}={api_key}"
        replacement = (
            f"{api_key_config.apikey_name}={create_secret_replacement(api_key)}"
        )
        mapping.append((query_param, replacement))
    return mapping


def create_secret_replacement(secret: str) -> str:
    return f"SCRAMBLED{hashlib.md5((secret + SECRET_SALT).encode(encoding='utf-8')).hexdigest()}"


def replace_secrets(mapping: SecretMapping, line: str) -> str:
    for secret, replacement in mapping:
        line = line.replace(secret, replacement)
    return line
